@@ -2,35 +2,27 @@
 ##'
 ##' I am resisting the urge to actually create the package here.
 ##' There are better options than I can come up with; for example
-##' \code{devtools::create}, \code{pkgkitten::kitten},
-##' \code{mason::mason}, or creating \code{DESCRIPTION} files using
-##' \code{desc}.  What is required here is that your package:
+##' `devtools::create`, `pkgkitten::kitten`, `mason::mason`, or
+##' creating `DESCRIPTION` files using `desc`.  What is required here
+##' is that your package:
 ##'
-##' \itemize{
-##' \item{Lists \code{odin} in \code{Imports:}}
-##' \item{Includes \code{useDynLib{<your package name>}} in
-##'   \code{NAMESPACE} (possibly via a roxygen comment \code{@useDynLib
-##'   <your package name>}}
-##' \item{To avoid a NOTE in \code{R CMD check}, import something from
-##'   \code{odin} in your namespace (e.g., \code{importFrom("odin", "odin")}
-##'   or roxygen \code{@importFrom(odin, odin)}}
-##' }
+##' * Lists `odin` in `Imports:`
+##' * Includes `useDynLib(<your package name>)` in
+##'   `NAMESPACE` (possibly via a roxygen comment `@useDynLib
+##'   <your package name>`
+##' * To avoid a NOTE in `R CMD check`, import something from
+##'   `odin` in your namespace (e.g., `importFrom("odin", "odin")`s
+##'   or roxygen `@importFrom(odin, odin)`
 ##'
 ##' Point this function at the package root (the directory containing
-##' \code{DESCRIPTION} and it will write out files \code{src/odin.c}
-##' and \code{odin.R}.  These files will be overwritten without
+##' `DESCRIPTION` and it will write out files `src/odin.c`
+##' and `odin.R`.  These files will be overwritten without
 ##' warning by running this again.
 ##'
-##' There are a few unresolved issues with this approach, notably
-##' activating "native symbol registration", and the interaction with
-##' packages such as Rcpp that automatically collate a list of
-##' symbols.  The mechanism may change in a future version, though the
-##' interface (with source files in \code{inst/odin} will remain the
-##' same.
-##'
 ##' @title Create odin model in a package
+##'
 ##' @param path_package Path to the package root (the directory that
-##'   contains \code{DESCRIPTION})
+##'   contains `DESCRIPTION`)
 ##'
 ##' @export
 ##' @examples
@@ -75,13 +67,18 @@ odin_package <- function(path_package) {
   }
 
   options <- odin_options(target = "c")
+  template <- read_lines(odin_file("template/odin_c.R"))
+
   f <- function(path) {
     ir <- odin_parse_(path, options)
     dat <- ir_deserialise(ir)
-    json <- sub("\\.[Rr]$", ".json", path)
-    writeLines(ir, json)
-    dat$ir <- sprintf("odin/%s", basename(json))
-    generate_c_code(dat, options, package)
+    path_json <- file.path(path_package, "inst", "odin",
+                           sprintf("%s.json", dat$config$base))
+    writeLines(ir, path_json)
+    ret <- generate_c_code(dat, options, package)
+    data <- wrapper_substitutions(dat, ret, package)
+    ret$r <- glue_whisker(template, data)
+    ret
   }
 
   dat <- lapply(filenames, f)
@@ -112,7 +109,8 @@ odin_package <- function(path_package) {
     }
   }
 
-  header <- "Automatically generated by odin - do not edit"
+  header <- sprintf("Automatically generated by odin %s - do not edit",
+                    as.character(utils::packageVersion("odin")))
 
   code_c <- c(
     paste("//", header),
@@ -120,18 +118,22 @@ odin_package <- function(path_package) {
     c_flatten_eqs(ring$declarations),
     c_flatten_eqs(interpolate$declarations),
     c_flatten_eqs(lapply(dat, "[[", "struct")),
-    c_flatten_eqs(lapply(dat, function(x) x$core$declaration)),
+    c_flatten_eqs(lapply(dat, function(x) x$code$declaration)),
     unname(lib$declarations),
     c_flatten_eqs(include$declarations),
-    c_flatten_eqs(lapply(dat, function(x) x$core$definition)),
+    c_flatten_eqs(lapply(dat, function(x) x$code$definition)),
     c_flatten_eqs(strsplit(lib$definitions, "\n")),
     c_flatten_eqs(ring$definitions),
     c_flatten_eqs(interpolate$definitions),
     c_flatten_eqs(include$definitions))
+
   code_r <- c(paste("##", header), c_flatten_eqs(lapply(dat, "[[", "r")))
 
   dir.create(file.path(path_package, "R"), FALSE)
   dir.create(file.path(path_package, "src"), FALSE)
   writeLines(code_c, file.path(path_package, "src", "odin.c"))
   writeLines(code_r, file.path(path_package, "R", "odin.R"))
+
+  path_reg <- file.path(path_package, "src/registration.c")
+  tools::package_native_routine_registration_skeleton(path_package, path_reg)
 }
